#include "Localisation.h"
#include "Commande_vitesse.h"
#include "math.h"

#define GAIN_P_POSITION 1
#define GAIN_P_ORIENTATION 10

struct position_t position_maintien;

/// @brief Asservissement de la position du robot. Les gains sont déterminés pour des positions très proches du robot
/// C'est à la consigne d'être défini avant pour être atteignable.
/// Nécessite l'appel des fonctions QEI_update(); Localisation_gestion(); AsserMoteur_Gestion(_step_ms);
/// @param position_consigne : position à atteindre dans le référentiel de la table.
void Asser_Position(struct position_t position_consigne){
    float delta_avance_mm;
    float avance_mm_s, rotation_radian_s;
    float delta_x_mm, delta_y_mm, delta_orientation_radian;
    struct position_t position_actuelle;

    position_actuelle = Localisation_get();

    // Calcul de l'erreur
    delta_x_mm = position_consigne.x_mm - position_actuelle.x_mm;
    delta_y_mm = position_consigne.y_mm - position_actuelle.y_mm;
    delta_avance_mm = sqrtf(delta_x_mm * delta_x_mm + delta_y_mm * delta_y_mm);
    delta_orientation_radian = atan2f(delta_y_mm, delta_x_mm) - position_actuelle.angle_radian;
    delta_orientation_radian = Geometrie_get_angle_optimal(0. , delta_orientation_radian);
    /*printf(">delta_orientation_radian:%.2f\n>angle_opti:%.2f\n>actuel:%.2f\n",delta_orientation_radian,
        Geometrie_get_angle_optimal(atan2f(delta_y_mm, delta_x_mm), position_actuelle.angle_radian),
        position_actuelle.angle_radian);
    printf(">pos_x:%.2f\n>pos_y:%.2f\n", position_actuelle.x_mm, position_actuelle.y_mm);
    printf(">con_x:%.2f\n>con_y:%.2f\n", position_consigne.x_mm, position_consigne.y_mm);*/

    // Asservissement
    avance_mm_s = delta_avance_mm * GAIN_P_POSITION;
    rotation_radian_s = delta_orientation_radian * GAIN_P_ORIENTATION;
    
    // Commande en vitesse
    commande_vitesse(avance_mm_s, rotation_radian_s);

}

void Asser_Position_set_Pos_Maintien(struct position_t position){
    position_maintien=position;
}

void Asser_Position_maintien(){
    Asser_Position(position_maintien);
}